# Vety语言入门指南

## 简介

Vety是一门现代化的编程语言，旨在提供简洁、高效且安全的编程体验。它结合了静态类型系统的安全性和动态语言的灵活性，使开发者能够快速构建可靠的应用程序。

## 设计理念

- **简洁性**：语法简单直观，减少不必要的复杂性
- **类型安全**：强大的静态类型系统，在编译时捕获潜在错误
- **性能优先**：高效的运行时性能，适合构建各类应用
- **原生集成**：良好的原生函数支持，易于与现有系统集成
- **模块化**：强大的模块系统，支持代码复用和组织

## 快速开始

### 1. 基本语法

```vety
// 变量声明
let message: string = "Hello, Vety!"
let number: i32 = 42

// 函数定义
func add(a: i32, b: i32): i32 {
    return a + b
}

// 条件语句
if (number > 0) {
    print("Positive number")
} else {
    print("Non-positive number")
}

// 循环
for (i: i32 = 0; i < 5; i++) {
    print(i)
}
```

### 2. 类型系统

Vety提供了丰富的内置类型：

- **基本类型**：
  - `i32`：32位整数
  - `i64`：64位整数
  - `f64`：64位浮点数
  - `bool`：布尔值
  - `string`：字符串
  - `void`：无返回值

- **复合类型**：
  - `array<T>`：数组
  - `map<K, V>`：映射
  - 自定义结构体

### 3. 错误处理

Vety使用try-catch机制处理错误：

```vety
try {
    let result = some_risky_operation()
} catch(e) {
    print("操作失败")
}
```

### 4. 模块系统

```vety
// 导入标准库模块
import io
import math

// 使用模块功能
let random_number = math.random(1, 100)
io.print("随机数：" + random_number)
```

### 5. 内置函数

Vety提供了一系列实用的内置函数：

- `print()`：输出信息
- `read_line()`：读取用户输入
- `len()`：获取集合长度
- `type_of()`：获取值的类型

## 最佳实践

1. **命名规范**
   - 变量和函数使用小写字母和下划线
   - 类型名使用大驼峰命名法

2. **代码组织**
   - 相关功能放在同一模块中
   - 适当使用注释说明代码功能

3. **错误处理**
   - 合理使用try-catch处理异常
   - 提供有意义的错误信息

4. **性能优化**
   - 避免不必要的内存分配
   - 合理使用循环和递归

## 下一步

- 阅读更详细的[语法指南](01_basic_syntax.md)
- 了解[控制流](02_control_flow.md)
- 学习[函数和模块](03_functions_and_modules.md)
- 探索[复合类型](04_composite_types.md)

## 示例项目

查看`demo`目录中的示例代码，了解更多Vety语言的实际应用：

- `hello.vt`：基本语法示例
- `array.vt`：数组操作示例
- `function.vt`：函数使用示例
- `try-catch.vt`：错误处理示例

## 贡献

Vety是一个开源项目，我们欢迎社区贡献。如果你发现了bug或有改进建议，请提交issue或pull request。

## 许可证

Vety使用MIT许可证，详细信息请查看LICENSE文件。