# Vety语言复合数据类型

## 1. 数组（Array）

### 1.1 数组声明

数组是相同类型元素的有序集合，使用方括号`[]`表示：

```vety
// 显式类型声明
let numbers: array = [1, 2, 3, 4, 5]

// 空数组
let empty_array: array = []

// 类型推断
let fruits = ["apple", "banana", "orange"]
```

### 1.2 数组访问

使用索引访问数组元素（索引从0开始）：

```vety
let numbers = [10, 20, 30, 40, 50]

// 访问元素
let first = numbers[0]    // 10
let second = numbers[1]   // 20

// 修改元素
numbers[2] = 35          // [10, 20, 35, 40, 50]
```

### 1.3 数组操作

```vety
// 获取数组长度
let length = numbers.length

// 遍历数组
for num in numbers {
    print(num)
}

// 使用索引遍历
for (let i = 0; i < numbers.length; i++) {
    print(numbers[i])
}
```

## 2. 映射（Map）

### 2.1 映射声明

映射是键值对的集合，键和值可以是不同类型：

```vety
// 显式类型声明
let scores: map<string, i32> = {
    "Alice": 95,
    "Bob": 87,
    "Charlie": 92
}

// 空映射
let empty_map: map<string, bool> = {}

// 类型推断
let config = {
    "debug": true,
    "port": 8080,
    "host": "localhost"
}
```

### 2.2 映射访问

使用键访问映射中的值：

```vety
let scores = {"Alice": 95, "Bob": 87}

// 访问值
let alice_score = scores["Alice"]    // 95

// 修改值
scores["Bob"] = 90                  // {"Alice": 95, "Bob": 90}

// 添加新键值对
scores["Charlie"] = 88              // {"Alice": 95, "Bob": 90, "Charlie": 88}
```

### 2.3 映射操作

```vety
// 检查键是否存在
if "Alice" in scores {
    print("Found Alice's score")
}

// 遍历映射
for key in scores {
    let value = scores[key]
    print(key + ": " + value)
}
```

## 3. 复合类型嵌套

数组和映射可以相互嵌套，创建更复杂的数据结构：

### 3.1 数组的数组

```vety
// 二维数组
let matrix: array<array<i32>> = [
    [1, 2, 3],
    [4, 5, 6],
    [7, 8, 9]
]

// 访问嵌套元素
let value = matrix[1][2]    // 6
```

### 3.2 映射的数组

```vety
// 用户列表
let users: array<map<string, string>> = [
    {"name": "Alice", "email": "alice@example.com"},
    {"name": "Bob", "email": "bob@example.com"}
]

// 访问嵌套数据
let first_user_email = users[0]["email"]
```

### 3.3 映射的映射

```vety
// 嵌套配置
let config: map<string, map<string, string>> = {
    "database": {
        "host": "localhost",
        "port": "5432",
        "name": "mydb"
    },
    "server": {
        "host": "0.0.0.0",
        "port": "8080"
    }
}

// 访问嵌套值
let db_host = config["database"]["host"]
```

## 4. 最佳实践

- 选择合适的数据结构
- 注意类型一致性
- 处理边界情况
- 合理使用嵌套

```vety
// 好的实践示例
func process_scores(scores: map<string, i32>): f64 {
    if scores.length == 0 {
        return 0.0
    }
    
    let total: i32 = 0
    for name in scores {
        total = total + scores[name]
    }
    return total / scores.length
}
```